<?php

/**
* 
*/
class sqlmap
{
	/**
	 * System error control
	 */
	
	private $error;
	private $error_msg;
	
	/**
	 * Static configuration
	 */
	
	private $cmd;
	private $host;
	private $filename;
	private $outputdir;
	
	/**
	 * Options:
	 */
	
	private $verbose;
	
	/**
	 * Target:
	 * 	At least one of these options has to be provided to define the
	 *  target(s)
	 */
	
	private $url;
	
	/**
	 * Request:
	 * 	These options can be used to specify how to connect to the target URL
	 */
	
	private $data;
	private $timeout;
	private $retries;
	
	/**
	 * Optimization:
	 * 	These options can be used to optimize the performance of sqlmap
	 */
	
	private $threads;
	
	/**
	 * Injection:
	 * 	These options can be used to specify which parameters to test for,
	 * 	provide custom injection payloads and optional tampering scripts
	 */
	
	/**
	 * Detection:
	 * 	These options can be used to customize the detection phase
	 */
	
	private $level;
	private $risk;
	
	/**
	 * Techniques:
	 * 	These options can be used to tweak testing of specific SQL injection
	 * 	techniques
	 */
	
	private $tech;
	private $timesec;
	private $unioncols;
	
	/**
	 * Fingerprint:
	 */
	
	/**
	 * Enumeration:
	 * 	These options can be used to enumerate the back-end database
	 * 	management system information, structure and data contained in the
	 * 	tables. Moreover you can run your own SQL statements
	 */
	
	private $currentuser;
	private $currentdb;
	private $hostname;
	private $users;
	private $privileges;
	private $dbs;
	private $tables;
	private $columns;
	private $schema;
	private $dump;
	
	/**
	 * Brute force:
	 * 	These options can be used to run brute force checks
	 */
	
	/**
	 * User-defined function injection:
	 * 	These options can be used to create custom user-defined functions
	 */
	
	/**
	 * File system access:
	 * 	These options can be used to access the back-end database management
	 * 	system underlying file system
	 */
	
	/**
	 * Operating system access:
	 * 	These options can be used to access the back-end database management
	 * 	system underlying operating system
	 */
	
	/**
	 * Windows registry access:
	 * 	These options can be used to access the back-end database management
	 * 	system Windows registry
	 */
	
	/**
	 * General:
	 * 	These options can be used to set some general working parameters
	 */
	
	// private $csvdel;
	// private $dumpdata;
	
	/**
	 * Miscellaneous:
	 */

	/**
	 * Functions of Class
	 */
	
	public function __construct($sqlmapdir, $outputdir)
	{
		$this->error 	= 0;
		$this->error_msg= "";

		$this->cmd 		= $sqlmapdir."/sqlmap.py";
		$this->host 	= "";
		$this->filename = "";
		$this->outputdir= " --output-dir=".$outputdir;

		$this->verbose 	= array('value' => 1, 'min' => 0, 'max' => 6);

		$this->url 		= "";

		$this->data 	= "";
		$this->randagent= 'off';
		$this->timeout 	= array('value' => 30, 'min' => 0, 'max' => 60);
		$this->retries 	= array('value' => 3, 'min' => 0, 'max' => 6);

		$this->threads 	= array('value' => 1, 'min' => 0, 'max' => 6);

		$this->level 	= array('value' => 1, 'min' => 1, 'max' => 5);
		$this->risk 	= array('value' => 1, 'min' => 0, 'max' => 3);

		$this->tech 	= array('B','E','U','S','T','Q');
		$this->timesec 	= array('value' => 5, 'min' => 0, 'max' => 6);
		$this->unioncols= "";

		$this->currentuser 	= 'off';
		$this->currentdb 	= 'off';
		$this->hostname 	= 'off';
		$this->users 		= 'off';
		$this->privileges 	= 'off';
		$this->dbs 			= 'off';
		$this->tables 		= 'off';
		$this->columns 		= 'off';
		$this->schema 		= 'off';
		$this->dump 		= 'off';

		// $this->csvdel 	= ",";
		// $this->dumpdata = "CSV";
	}

	public function __destruct()
	{
		# code...
	}

	/**
	 * SYSTEM ERROR CONTROL
	 */

	public function preventSQLInjection($str)
	{
		return mysql_real_escape_string(stripslashes($str));
	}

	public function errorSettings()
	{ 
		if ($this->error == 1) {
			return true;
		}
		return false;
	}

	public function setErrorMessage($str)
	{
		$this->error = 1;
		// $message = preventSQLInjection($message);
		$this->error_msg .= "[ERROR] ". $str . "<br>";
	}

	public function getErrorMessage()
	{
		return $this->error_msg;
	}

	/**
	 * STATIC CONFIGURATION
	 */

	private function setHost()
	{
		$ipos = strpos($this->url, "/") + 2;
		$fpos = strpos($this->url, "/", $ipos) - $ipos;
		$this->host = substr($this->url, $ipos, $fpos);
	}

	public function getHost()
	{
		return $this->host;
	}

	public function setFilename()
	{
		$timestamp = date("Ymd-Hi");
		$this->filename = "report-sqlmap-".$this->host."-".$timestamp.".txt";
	}

	public function getFilename()
	{
		//setFilename();
		return $this->filename;
	}

	/**
	 * VARIABLE CONFIGURATION
	 */

	public function setSettings($value)
	{
		$this->cmd .= $value;
	}

	public function getSQLmapInstruction()
	{
		// $this->cmd .= ' --answers="quit=Y,follow=Y" --batch';
		$this->cmd .= ' --batch';
		$this->cmd .= $this->outputdir;
		return $this->cmd;
	}

	/**
	 * COMMON FUNCTIONS
	 */
	
	public function checkArrayValue($parameter)
	{
		$object = 'sqlmap';
		$get_func = "get".ucfirst($parameter);
		$set_func = "set".ucfirst($parameter);

		if (isset($_POST[$parameter])) {
			$aux = call_user_func(array($object,$get_func));
			if ($aux['value'] != $_POST[$parameter]) {
				call_user_func(array($object,$set_func),$_POST[$parameter]);
			}
		}
	}

	public function checkActiveParameter($parameter)
	{
		$object = 'sqlmap';
		$get_func = "get".ucfirst($parameter);
		$set_func = "set".ucfirst($parameter);

		if (isset($_POST[$parameter])) {
			// $value = call_user_func(array($object,$get_func));
			if (strcmp('on', $_POST[$parameter]) == 0) {
				call_user_func(array($object,$set_func),$_POST[$parameter]);
			} else {
				$msg = "The '".$parameter."' option only allows two parameters: enable or disable.";
				$this->setErrorMessage($msg);
				return -1;
			}
		}
	}

	private function setValueRange($parameter, $value, $min, $max)
	{
		$var = $this->$parameter;
		$success = false;

		/*
		echo $value."<br>";
		$value = -1;
		echo $value."<br>";
		*/

		if ($value >= $min && $value <= $max) {
			$var['value'] = $value;
			$success = true;
		}

		switch ($parameter) {
			case 'verbose':
				if (!$success) {
					$msg = "Verbosity level must be between ".$min." and ".$max." and the given parameter is '".$value."'.";
				}
				$str = " -v " . $value;
				break;

			case 'timeout':
				if (!$success) {
					$msg = "Timeout connection must be between ".$min." and ".$max." and the given parameter is '".$value."'.";
				}
				$str = " --timeout=" . $value;
				break;
			
			case 'retries':
				if (!$success) {
					$msg = "The number of retries must be between ".$min." and ".$max." and the given parameter is '".$value."'.";
				}
				$str = " --retries=" . $value;
				break;

			case 'threads':
				if (!$success) {
					$msg = "The number of threads must be between ".$min." and ".$max." and the given parameter is '".$value."'.";
				}
				$str = " --threads=" . $value;
				break;

			case 'level':
				if (!$success) {
					$msg = "Level of tests to perform must be between ".$min." and ".$max." and the given parameter is '".$value."'.";
				}
				$str = " --level=" . $value;
				break;

			case 'risk':
				if (!$success) {
					$msg = "Risk of tests to perform must be between ".$min." and ".$max." and the given parameter is '".$value."'.";
				}
				$str = " --risk=" . $value;
				break;

			case 'timesec':
				if (!$success) {
					$msg = "Seconds to delay the DBMS response must be between ".$min." and ".$max." and the given parameter is '".$value."'.";
				}
				$str = " --time-sec=" . $value;
				break;

			default:
				$msg = "The parameter entered ['".$parameter."'] doesn't exist.";
				$this->setErrorMessage($msg);
				return -1;
				break;
		}
		if (!$success) {
			$this->setErrorMessage($msg);
			return -1;
		}
		$this->setSettings($str);		
	}

	/**
	 * OPTIONS
	 */

	public function setVerbose($value)
	{
		$parameter = 'verbose';
		$var = $this->$parameter;

		$this->setValueRange($parameter,$value,$var['min'],$var['max']);
	}

	public function getVerbose()
	{
		return $this->verbose;
	}

	/**
	 * TARGET
	 */

	public function setURL($value)
	{
		$pos = strpos($value, "?");
		if ($pos != false) {
			$str = substr($value, 0, $pos);
		} else {
			$str = $value;
		}
	
		$file_headers = @get_headers($str);
		
		if (strpos($file_headers[0], "200 OK") != false) {
			$this->url = $value;
			$this->setHost();
		} else {
			$msg = "The URL ('".$value."') doesn't exist.";
			$this->setErrorMessage($msg);
			return -1;
		}
		$str = ' -u "' . $value . '"';
		$this->setSettings($str);
	}

	public function getURL()
	{
		return $this->url;
	}

	/**
	 * REQUEST
	 */
	
	public function setData($value)
	{
		$this->data = $value;
		$str = ' --data="'.$value.'"';
		$this->setSettings($str);
	}
	
	public function getData()
	{
		return $this->data;
	}

	private function setRandagent($value)
	{
		$this->randagent= 'on';
		$str = ' --random-agent';
		$this->setSettings($str);
	}

	public function getRandagent()
	{
		return $this->randagent;
	}

	private function setTimeout($value)
	{
		$parameter = 'timeout';
		$var = $this->$parameter;

		$this->setValueRange($parameter,$value,$var['min'],$var['max']);
	}

	public function getTimeout()
	{
		return $this->timeout;
	}

	private function setRetries($value)
	{
		$parameter = 'retries';
		$var = $this->$parameter;

		$this->setValueRange($parameter,$value,$var['min'],$var['max']);
	}

	public function getRetries()
	{
		return $this->retries;
	}

	/**
	 * OPTIMIZATION
	 */
	
	private function setThreads($value)
	{
		$parameter = 'threads';
		$var = $this->$parameter;

		$this->setValueRange($parameter,$value,$var['min'],$var['max']);
	}

	public function getThreads()
	{
		return $this->threads;
	}

	/**
	 * DETECTION
	 */
	
	private function setLevel($value)
	{
		$parameter = 'level';
		$var = $this->$parameter;

		$this->setValueRange($parameter,$value,$var['min'],$var['max']);
	}

	public function getLevel()
	{
		return $this->level;
	}

	private function setRisk($value)
	{
		$parameter = 'risk';
		$var = $this->$parameter;

		$this->setValueRange($parameter,$value,$var['min'],$var['max']);
	}

	public function getRisk()
	{
		return $this->risk;
	}

	/**
	 * TECHNIQUES
	 */
	
	public function setTech($variable)
	{
		$same_length = false;
		$aux = "";

		if (count($variable) == count($this->tech)) {
			$same_length = true;
		}

		foreach ($variable as $value) {
			if (in_array($value, $this->tech)) {
				$aux .= $value;
			} else {
				$msg = "The SQL injection technique ('".$value."') doesn't exist.";
				$this->setErrorMessage($msg);
				return -1;
			}
		}

		if (!$same_length) {
			$str = " --technique=" . implode($variable);
			$this->setSettings($str);
		}
	}

	public function getTech()
	{
		return $this->tech;
	}

	private function setTimesec($value)
	{
		$parameter = 'timesec';
		$var = $this->$parameter;

		$this->setValueRange($parameter,$value,$var['min'],$var['max']);
	}

	public function getTimesec()
	{
		return $this->timesec;
	}

	public function setUnioncols($value)
	{
		$this->unioncols = $value;
		$str = ' --union-cols="'.$value.'"';
		$this->setSettings($str);
	}
	
	public function getUnioncols()
	{
		return $this->unioncols;
	}

	/**
	 * ENUMERATION
	 */
	
	private function setCurrentuser($value)
	{
		$this->currentuser= 'on';
		$str = ' --current-user';
		$this->setSettings($str);
	}

	public function getCurrentuser()
	{
		return $this->currentuser;
	}

	private function setCurrentdb($value)
	{
		$this->currentdb= 'on';
		$str = ' --current-db';
		$this->setSettings($str);
	}

	public function getCurrentdb()
	{
		return $this->currentdb;
	}

	private function setHostname($value)
	{
		$this->hostname= 'on';
		$str = ' --hostname';
		$this->setSettings($str);
	}

	public function getHostname()
	{
		return $this->hostname;
	}

	private function setUsers($value)
	{
		$this->users= 'on';
		$str = ' --users';
		$this->setSettings($str);
	}

	public function getUsers()
	{
		return $this->users;
	}

	private function setPrivileges($value)
	{
		$this->privileges= 'on';
		$str = ' --privileges';
		$this->setSettings($str);
	}

	public function getPrivileges()
	{
		return $this->privileges;
	}

	private function setDbs($value)
	{
		$this->dbs= 'on';
		$str = ' --dbs';
		$this->setSettings($str);
	}

	public function getDbs()
	{
		return $this->dbs;
	}

	private function setTables($value)
	{
		$this->tables= 'on';
		$str = ' --tables';
		$this->setSettings($str);
	}

	public function getTables()
	{
		return $this->tables;
	}

	private function setColumns($value)
	{
		$this->columns= 'on';
		$str = ' --columns';
		$this->setSettings($str);
	}

	public function getColumns()
	{
		return $this->columns;
	}

	private function setSchema($value)
	{
		$this->schema= 'on';
		$str = ' --schema';
		$this->setSettings($str);
	}

	public function getSchema()
	{
		return $this->schema;
	}

	private function setDump($value)
	{
		$this->dump= 'on';
		$str = ' --dump';
		$this->setSettings($str);
	}

	public function getDump()
	{
		return $this->dump;
	}

	/**
	 * GENERAL
	 */
	
}

?>